@@ -111,10 +111,12 @@ class AnnotationResult():
                 self.adata.obs[f"{prefix}conf_score"] = self.probability_matrix.max(axis=1).values
             elif insert_conf_by == 'majority_voting':
                 if insert_conf_by not in self.predicted_labels:
-                    raise KeyError(f"🛑 Did not find the column `majority_voting` in the `AnnotationResult.predicted_labels`, perform majority voting beforehand or use `insert_conf_by = 'predicted_labels'` instead")
+                    raise KeyError(
+                            f"🛑 Did not find the column `majority_voting` in the `AnnotationResult.predicted_labels`, perform majority voting beforehand or use `insert_conf_by = 'predicted_labels'` instead")
                 self.adata.obs[f"{prefix}conf_score"] = [row[self.predicted_labels.majority_voting[index]] for index, row in self.probability_matrix.iterrows()]
             else:
-                raise KeyError(f"🛑 Unrecognized `insert_conf_by` value ('{insert_conf_by}'), should be one of `'predicted_labels'` or `'majority_voting'`")
+                raise KeyError(
+                        f"🛑 Unrecognized `insert_conf_by` value ('{insert_conf_by}'), should be one of `'predicted_labels'` or `'majority_voting'`")
         if insert_prob:
             self.adata.obs[[f"{prefix}{x}" for x in self.probability_matrix.columns]] = self.probability_matrix
         elif insert_decision:
@@ -149,7 +151,8 @@ class AnnotationResult():
             4) **name of each cell type**, which represents the decision scores and probabilities of a given cell type distributed across cells overlaid onto the UMAP.
         """
         if not os.path.isdir(folder):
-            raise FileNotFoundError(f"🛑 Output folder {folder} does not exist. Please provide a valid folder")
+            raise FileNotFoundError(
+                    f"🛑 Output folder {folder} does not exist. Please provide a valid folder")
         if 'X_umap' in self.adata.obsm:
             logger.info("👀 Detected existing UMAP coordinates, will plot the results accordingly")
         elif 'connectivities' in self.adata.obsp:
@@ -166,12 +169,14 @@ class AnnotationResult():
         for column in self.predicted_labels:
             sc.pl.umap(self.adata, color = column, legend_loc = 'on data', show = False, legend_fontweight = 'normal', title = column.replace('_', ' '))
             plt.savefig(os.path.join(folder, prefix + column + '.' + format))
+            plt.close()
         if plot_probability:
             for column in self.probability_matrix:
                 self.adata.obs['decision score'] = self.decision_matrix[column]
                 self.adata.obs['probability'] = self.probability_matrix[column]
                 sc.pl.umap(self.adata, color = ['decision score', 'probability'], show = False)
                 plt.savefig(os.path.join(folder, prefix + column.replace('/','_') + '.' + format))
+                plt.close()
             self.adata.obs.drop(columns=['decision score', 'probability'], inplace=True)
 
     def to_table(self, folder: str, prefix: str = '', xlsx: bool = False) -> None:
@@ -194,7 +199,8 @@ class AnnotationResult():
             Depending on `xlsx`, return table(s) of predicted labels, decision matrix and probability matrix.
         """
         if not os.path.isdir(folder):
-            raise FileNotFoundError(f"🛑 Output folder {folder} does not exist. Please provide a valid folder")
+            raise FileNotFoundError(
+                    f"🛑 Output folder {folder} does not exist. Please provide a valid folder")
         if not xlsx:
             self.predicted_labels.to_csv(os.path.join(folder, f"{prefix}predicted_labels.csv"))
             self.decision_matrix.to_csv(os.path.join(folder, f"{prefix}decision_matrix.csv"))
@@ -269,17 +275,20 @@ class Classifier():
                 self.adata = self.adata.transpose()
             if self.filename.endswith(('.mtx', '.mtx.gz')):
                 if (gene_file is None) or (cell_file is None):
-                    raise FileNotFoundError("🛑 Missing `gene_file` and/or `cell_file`. Please provide both arguments together with the input mtx file")
+                    raise FileNotFoundError(
+                            "🛑 Missing `gene_file` and/or `cell_file`. Please provide both arguments together with the input mtx file")
                 genes_mtx = pd.read_csv(gene_file, header=None)[0].values
                 cells_mtx = pd.read_csv(cell_file, header=None)[0].values
                 if len(genes_mtx) != self.adata.n_vars:
-                    raise ValueError(f"🛑 The number of genes in {gene_file} does not match the number of genes in {self.filename}")
+                    raise ValueError(
+                            f"🛑 The number of genes in {gene_file} does not match the number of genes in {self.filename}")
                 if len(cells_mtx) != self.adata.n_obs:
-                    raise ValueError(f"🛑 The number of cells in {cell_file} does not match the number of cells in {self.filename}")
+                    raise ValueError(
+                            f"🛑 The number of cells in {cell_file} does not match the number of cells in {self.filename}")
                 self.adata.var_names = genes_mtx
                 self.adata.obs_names = cells_mtx
             if not float(self.adata.X.max()).is_integer():
-                logger.warn(f"⚠️ Warning: the input file seems not a raw count matrix. The prediction result may be biased")
+                logger.warn(f"⚠️ Warning: the input file seems not a raw count matrix. The prediction result may not be accurate")
             if (self.adata.n_vars >= 100000) or (len(self.adata.var_names[0]) >= 30) or (len(self.adata.obs_names.intersection(['GAPDH', 'ACTB', 'CALM1', 'PTPRC', 'MALAT1'])) >= 1):
                 logger.warn(f"⚠️ The input matrix is detected to be a gene-by-cell matrix, will transpose it")
                 self.adata = self.adata.transpose()
@@ -292,22 +301,27 @@ class Classifier():
         elif isinstance(filename, AnnData) or (isinstance(filename, str) and filename.endswith('.h5ad')):
             self.adata = sc.read(filename) if isinstance(filename, str) else filename
             self.adata.var_names_make_unique()
-            if self.adata.X.min() < 0:
-                logger.info("👀 Detected scaled expression in the input data, will try the `.raw` attribute")
+            if (self.adata.X.min() < 0) or (self.adata.X.max() > np.log1p(10000)):
+                logger.info("👀 Invalid expression matrix in `.X`, expect log1p normalized expression to 10000 counts per cell; will try the `.raw` attribute")
                 try:
                     self.indata = self.adata.raw.X
                     self.indata_genes = self.adata.raw.var_names
                     self.indata_names = self.adata.raw.obs_names
                 except Exception as e:
-                    raise Exception(f"🛑 Fail to use the `.raw` attribute in the input object. {e}")
+                    raise Exception(
+                            f"🛑 Fail to use the `.raw` attribute in the input object. {e}")
+                if (self.indata.min() < 0) or (self.indata.max() > np.log1p(10000)):
+                    raise ValueError(
+                            "🛑 Invalid expression matrix in both `.X` and `.raw.X`, expect log1p normalized expression to 10000 counts per cell")
             else:
                 self.indata = self.adata.X
                 self.indata_genes = self.adata.var_names
                 self.indata_names = self.adata.obs_names
             if np.abs(np.expm1(self.indata[0]).sum()-10000) > 1:
-                raise ValueError("🛑 Invalid expression matrix, expect log1p normalized expression to 10000 counts per cell")
+                logger.warn(f"⚠️ Warning: invalid expression matrix, expect all genes and log1p normalized expression to 10000 counts per cell. The prediction result may not be accurate")
         else:
-            raise ValueError("🛑 Invalid input. Supported types: .csv, .txt, .tsv, .tab, .mtx, .mtx.gz and .h5ad, or AnnData loaded in memory")
+            raise ValueError(
+                    "🛑 Invalid input. Supported types: .csv, .txt, .tsv, .tab, .mtx, .mtx.gz and .h5ad, or AnnData loaded in memory")
 
         logger.info(f"🔬 Input data has {self.indata.shape[0]} cells and {len(self.indata_genes)} genes")
 
@@ -338,7 +352,8 @@ class Classifier():
         logger.info(f"🔗 Matching reference genes in the model")
         k_x = np.isin(self.indata_genes, self.model.classifier.features)
         if k_x.sum() == 0:
-            raise ValueError(f"🛑 No features overlap with the model. Please provide gene symbols")
+            raise ValueError(
+                    f"🛑 No features overlap with the model. Please provide gene symbols")
         else:
             logger.info(f"🧬 {k_x.sum()} features used for prediction")
         k_x_idx = np.where(k_x)[0]
@@ -347,7 +362,7 @@ class Classifier():
         lr_idx = pd.DataFrame(self.model.classifier.features, columns=['features']).reset_index().set_index('features').loc[self.indata_genes, 'index'].values
 
         logger.info(f"⚖️ Scaling input data")
-        means_ = self.model.scaler.mean_[lr_idx]
+        means_ = self.model.scaler.mean_[lr_idx] if self.model.scaler.with_mean else 0
         sds_ = self.model.scaler.scale_[lr_idx]
         self.indata = (self.indata[:, k_x_idx] - means_) / sds_
         self.indata[self.indata > 10] = 10
